using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace Escape;

/// @author annilehtinen
/// @version 30.03.2025 / 3
/// <summary>
/// Peli, jossa pelaajaan tavoitteena on vältellä vastustajaa (Barkov) ja kerätä kiekkoja saadakseen pisteitä.
/// </summary>
public class Escape : PhysicsGame
{
    public override void Begin()
    {
        Camera.ZoomToLevel();
        CreateLevel();
        
        PhysicsObject player = AddPlayer(this);
        SetControls(player);

        PhysicsObject barkov = AddBarkov(this, player);
        AddCollisionHandler(player, barkov, Tackle);
        
        AddCollisionHandler(player, "puck", CollectPuck);
        
        Timer.CreateAndStart(4.0, SpawnPucks);
    }
    
    
    /// <summary>
    /// Peliohjainten asettaminen pelaajalle
    /// </summary>
    /// <param name="player">Hahmo, jolle ohjainkomennot lisätään</param>
    void SetControls(PhysicsObject player)
    {
        Keyboard.Listen(Key.Up, ButtonState.Down, MovePlayer, "Liikuta pelaajaa eteenpäin", player);
        Keyboard.Listen(Key.Left, ButtonState.Down, TurnPlayer, "Käännä pelaajaa vasemmalle", player, 2.0);
        Keyboard.Listen(Key.Left, ButtonState.Released, TurnPlayer, null, player, 0.0);
        Keyboard.Listen(Key.Right, ButtonState.Down, TurnPlayer, "Käännä pelaajaa oikealle", player,-2.0);
        Keyboard.Listen(Key.Right, ButtonState.Released, TurnPlayer, null, player,0.0);
        
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    } 
    
    
    /// <summary>
    /// Lisää pelaajan hahmon pelikentälle ja määrittää sen ominaisuudet
    /// </summary>
    /// <param name="game">Peli, johon hahmo lisätään</param>
    /// <returns>Pelaaja</returns>
    static PhysicsObject AddPlayer(PhysicsGame game)
    {
        PhysicsObject player = new PhysicsObject(40, 40);
        player.Image = LoadImage("player.png");
        player.X = -450;
        player.Y = 0;
        player.CanRotate = false;
        player.LinearDamping = 0.95;
        
        game.Add(player);
        return player;
    } 
    
    
    /// <summary>
    /// Lisää "vihollisen" eli Barkovin pelikentälle
    /// </summary>
    /// <param name="game">Peli, johon hahmo lisätään</param>
    /// <param name="target">Kohde, jota Barkov seuraa (pelaajan hahmo)</param>
    /// <returns>Barkov</returns>
    static PhysicsObject AddBarkov(PhysicsGame game, PhysicsObject target)
    {
        PhysicsObject barkov = new PhysicsObject(50, 50);
        barkov.Image = LoadImage("Barkov.png");
        barkov.X = 450;
        barkov.Y = 0;
        
        FollowerBrain barkovBrain = new FollowerBrain(target);
        barkovBrain.DistanceFar = 600;
        barkovBrain.Speed = 50;
        barkovBrain.Active = true;
        barkovBrain.TurnWhileMoving = true;
        barkov.Brain = barkovBrain;
        
        game.Add(barkov);
        return barkov;
    } 
    
    
    /// <summary>
    /// Törmäyksenkäsittelijä pelaajahahmolle ja Barkoville (kesken)
    /// </summary>
    /// <param name="player">Pelaajan hahmo</param>
    /// <param name="barkov">Barkov</param>
    void Tackle(PhysicsObject player, PhysicsObject barkov)
    {
        MessageDisplay.Add("Taklaus");
        //Peli päättyy tähän taklaukseen
    } 
    
    
    /// <summary>
    /// Pelaaja kerää kiekon pelikentältä
    /// </summary>
    /// <param name="player">Pelaajan hahmo</param>
    /// <param name="puck">Kiekko, joka poistetaan kentältä</param>
    void CollectPuck(PhysicsObject player, PhysicsObject puck)
    {
        puck.Destroy();
    }
    
    
    /// <summary>
    /// Pelaajan liikuttaminen eteenpäin (myöhemmin animaation kanssa)
    /// </summary>
    /// <param name="player">Hahmo, jota liikutetaan</param>
    void MovePlayer(PhysicsObject player)
    {
        int playerSpeed = 10;
        Vector playerDirection = Vector.FromLengthAndAngle(50.0, player.Angle);
        player.Push(playerDirection * playerSpeed);
    } 
    
    
    /// <summary>
    /// Pelaajan kääntäminen nuolinäppäinten avulla
    /// </summary>
    /// <param name="player">Pelaaja, jonka hahmoa käännetään</param>
    /// <param name="angle">Kulma, jossa hahmoa käännetään</param>
    void TurnPlayer(PhysicsObject player, double angle)
    {
        player.Angle += Angle.FromDegrees(angle);
    } 
    
    
    /// <summary>
    /// Kiekon lisääminen kentälle satunnaiseen kohtaan
    /// </summary>
    /// <param name="amount">Kuinka monta kiekkoa luodaan kentälle</param>
    /// <param name="lifetime">Aika, jonka kiekko on kentällä ennen kuin se katoaa</param>
    void SpawnPucks(int amount, double lifetime)
    {
        for (int i = 1; i <= amount; i++)
        {
            PhysicsObject puck = new PhysicsObject(18, 18);
            puck.Shape = Shape.Circle;                         
            puck.Color = Color.Black;                          
            puck.LifetimeLeft = TimeSpan.FromSeconds(lifetime);
            puck.Position = Level.GetRandomPosition();         
            puck.Tag = "puck";                                 
            this.Add(puck);
        }
    } 
    
    
    /// <summary>
    /// Yhden kiekon lisääminen kentälle satunnaiseen kohtaan viiden sekunnin ajaksi,
    /// jos kutsussa ei anneta parametrinä määrää ja aikaa erikseen
    /// </summary>
    void SpawnPucks()
    {
        SpawnPucks(1, 5.0);
    }
    
    
    /// <summary>
    /// Lisätään kentälle haluttu määrä kiekkoja (pelin alkaessa)
    /// </summary>
    void CreateLevel()
    {
        Surfaces borders = Level.CreateBorders(0.5, true, Color.White);
        SpawnPucks(5, 10.0);
    }
}